/*
 Project name:
       LIN Hall Click
 Copyright:
      (c) mikroElektronika, 2014.
 Revision History:
      20140822:
      - Initial release (RD);
 Description:
      LIN HALL click is a simple solution for adding a linear Hall sensor to your design. 
      It carries an MLX90242 linear Hall sensor IC (with built-in active error correction circuitry) and an MCP3201 12-bit ADC. 
      LIN HALL click communicates with the target board through the mikroBUS SPI lines (CS, SCK, MISO). 
      The board is designed to use either a 3.3V or 5V power supply.
      LIN HALL click, with its linear Hall sensor, outputs a signal that is proportional to the flux density of a magnetic field. 
      The output signal will increase when a north magnetic field is applied, while a south magnetic field will cause it to decrease. 
      The ratiometric output can be used to measure the strength of different magnets, or to measure the distance between the sensor 
      and a magnet (which can be used for linear or rotary position sensing by placing magnets on the target object). LCD shows data.

 * Test configuration:
     MCU:             ATmega32
                      http://www.atmel.com/Images/doc2503.pdf
     Dev. Board:      EasyAVR7
                      http://www.mikroe.com/easyavr/
     Oscillator:      Oscillator Frequency 8.0000 MHz, 8.0000 MHz Crystal
     ext. modules:    BI Hall Click
                      http://www.mikroe.com/click/bi-hall/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/
 NOTES:
      - Place Lin Hall Click board in the mikroBUS socket 1.
      - Set development board J5 jumper according to SMD jumper on click board. By default it's soldered in the 3.3V position.
      - Turn On SW3-1 for LCD backlight, and SW5-3, SW5-4, SW5-5 for SPI.

*/

// HALL click module connections
sbit Chip_Select             at PORTA5_bit;
// end HALL click module connections

// Lcd module connections
sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End Lcd module connections

unsigned int measurement, lastValue;
char txt[10];


// Get ADC values
unsigned int getADC() {    // Returns 0..4095
  unsigned int tmp;
  Chip_Select = 0;                               // Select MCP3201
  tmp = SPI1_Read(0) & 0x1F;                     // Dummy 0 byte out , read in upper bits - B11-B7
  tmp = tmp << 7;                                // Shift ADC value by 7
  tmp += SPI1_Read(0) >> 1;                      // Dummy 0 byte out , read in lower  bits, B6-B0 + B1
                                                 // and shift right by 1 to determine B1
  Chip_Select= 1;                                // Deselect MCP3201
  return tmp;                                    // Returns 12-bit ADC value
}

void main() {
  Lcd_Init();                      // Initialize Lcd

  Lcd_Cmd(_LCD_CLEAR);             // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);        // Cursor off

  measurement = 0;                 // Initialize the measurement variable

  Chip_Select = 1;                 // Deselect MCP3201
  DDRA.B5 = 1;                     // Set chip select pin to be output

  // Initialize SPI1
  SPI1_Init();
  SPI_Set_Active(&SPI1_Read, &SPI1_Write);      // Sets the SPI1 module active

  while (1) {
    measurement = getADC();       // Get ADC result
    WordToStr(measurement,txt);
    Lcd_Out(1,1,txt);
    Delay_ms(200);                // Wait 200ms
  }
}